@@ -87,7 +87,7 @@ use Time::HiRes qw(time);
 use Cwd;
 use Config;
 
-our $VERSION = "3.004003";      # update after release (same format as perl $])
+our $VERSION = "3.004004";      # update after release (same format as perl $])
 #our $IS_DEVEL_BUILD = 1;        # 1 for devel build 
 our $IS_DEVEL_BUILD = 0;        # 0 for release versions including rc & pre releases
 
@@ -101,18 +101,18 @@ our @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
 our $SUB_VERSION = 'svnunknown';
-if ('$LastChangedDate: 2019-12-06 15:58:14 -0800 (Fri, 06 Dec 2019) $' =~ ':') {
-  # Subversion keyword "$LastChangedDate: 2019-12-06 15:58:14 -0800 (Fri, 06 Dec 2019) $" has been successfully expanded.
+if ('$LastChangedDate: 2020-01-24 18:49:19 -0800 (Fri, 24 Jan 2020) $' =~ ':') {
+  # Subversion keyword "$LastChangedDate: 2020-01-24 18:49:19 -0800 (Fri, 24 Jan 2020) $" has been successfully expanded.
   # Doesn't happen with automated launchpad builds:
   # https://bugs.launchpad.net/launchpad/+bug/780916
-  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2019-12-06 15:58:14 -0800 (Fri, 06 Dec 2019) $ updated by SVN'))[1];
+  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2020-01-24 18:49:19 -0800 (Fri, 24 Jan 2020) $ updated by SVN'))[1];
 }
 
 
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
-  if ('$LastChangedRevision: 1870940 $' =~ ':') {
-    # Subversion keyword "$LastChangedRevision: 1870940 $" has been successfully expanded.
-    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1870940 $ updated by SVN}[1]));
+  if ('$LastChangedRevision: 1873123 $' =~ ':') {
+    # Subversion keyword "$LastChangedRevision: 1873123 $" has been successfully expanded.
+    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1873123 $ updated by SVN}[1]));
   } else {
     push(@EXTRA_VERSION, ('r' . 'svnunknown'));
   }
