@@ -12,31 +12,36 @@ use File::Basename;
 use File::Copy;
 use File::Path;
 use File::Spec;
+
+use Test::Builder ();
+use Test::More    ();
+
 use POSIX qw(WIFEXITED WIFSIGNALED WIFSTOPPED WEXITSTATUS WTERMSIG WSTOPSIG);
 
+use vars qw($RUNNING_ON_WINDOWS $SSL_AVAILABLE
+            $SKIP_SPAMD_TESTS $SKIP_SPAMC_TESTS $NO_SPAMC_EXE
+            $SKIP_SETUID_NOBODY_TESTS $SKIP_DNSBL_TESTS
+            $have_inet4 $have_inet6 $spamdhost $spamdport);
+
 BEGIN {
   require Exporter;
   use vars qw(@ISA @EXPORT @EXPORT_OK);
   @ISA = qw(Exporter);
 
-  use vars qw($have_inet4 $have_inet6 $spamdhost $spamdport);
   @EXPORT = qw($have_inet4 $have_inet6 $spamdhost $spamdport);
 
   # No spamd test in Windows unless env override says user figured out a way
   # If you want to know why these are vars and no constants, read this thread:
   #   <http://www.mail-archive.com/dev%40perl.apache.org/msg05466.html>
   #  -- mss, 2004-01-13
-  our $RUNNING_ON_WINDOWS = ($^O =~ /^(mswin|dos|os2)/oi);
-  our $SKIP_SPAMD_TESTS =
+  $RUNNING_ON_WINDOWS = ($^O =~ /^(mswin|dos|os2)/oi);
+  $SKIP_SPAMD_TESTS =
         $RUNNING_ON_WINDOWS ||
         ( $ENV{'SPAMD_HOST'} && !($ENV{'SPAMD_HOST'} eq '127.0.0.1' ||
                                   $ENV{'SPAMD_HOST'} eq '::1' ||
                                   $ENV{'SPAMD_HOST'} eq 'localhost') );
-  our $NO_SPAMC_EXE;
-  our $SKIP_SPAMC_TESTS;
-  our $SSL_AVAILABLE;
-  our $SKIP_SETUID_NOBODY_TESTS = 0;
-  our $SKIP_DNSBL_TESTS = 0;
+  $SKIP_SETUID_NOBODY_TESTS = 0;
+  $SKIP_DNSBL_TESTS = 0;
 
   $have_inet4 = eval {
     require IO::Socket::INET;
@@ -51,6 +56,11 @@ BEGIN {
     $sock->close or die "error closing inet6 socket: $!"  if $sock;
     $sock ? 1 : undef;
   };
+
+  # Clean PATH so taint doesn't complain
+  $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
+  # Remove tainted envs, at least ENV used in FreeBSD
+  delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};
 }
 
 # Set up for testing. Exports (as global vars):
@@ -90,7 +100,7 @@ sub sa_t_init {
   $perl_cmd .= " -T" if !defined($ENV{'TEST_PERL_TAINT'}) or $ENV{'TEST_PERL_TAINT'} ne 'no';
   $perl_cmd .= " -w" if !defined($ENV{'TEST_PERL_WARN'})  or $ENV{'TEST_PERL_WARN'}  ne 'no';
 
-  $scr = $ENV{'SCRIPT'};
+  $scr = $ENV{'SPAMASSASSIN_SCRIPT'};
   $scr ||= "$perl_cmd ../spamassassin.raw";
 
   $spamd = "$perl_cmd ../spamd/spamd.raw";
@@ -159,15 +169,21 @@ sub sa_t_init {
                        ($RUNNING_ON_WINDOWS && !$ENV{'SPAMD_HOST'})); 
     $SSL_AVAILABLE = ((!$SKIP_SPAMC_TESTS) &&  # no SSL test if no spamc
                     (!$SKIP_SPAMD_TESTS) &&  # or if no local spamd
-                    (`$spamc -V` =~ /with SSL support/) &&
-                    (`$spamd --version` =~ /with SSL support/));
+                    (untaint_cmd("$spamc -V") =~ /with SSL support/) &&
+                    (untaint_cmd("$spamd --version") =~ /with SSL support/));
   }
   # do not remove prior test results!
   # rmtree ("log");
 
-  mkdir ("log", 0755);
+  unless (-d "log") {
+    mkdir ("log", 0755) or die ("Error creating log dir: $!");
+  }
   chmod (0755, "log"); # set in case log already exists with wrong permissions
 
+  if (!$RUNNING_ON_WINDOWS) {
+    untaint_system("chacl -B log 2>/dev/null || setfacl -b log 2>/dev/null"); # remove acls that confuse test
+  }
+
   rmtree ("log/user_state");
   rmtree ("log/outputdir.tmp");
 
@@ -234,7 +250,6 @@ sub probably_unused_spamd_port {
 
   my $port;
   my @nstat;
-  local $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';  # must not be tainted
   if (!open(NSTAT, "netstat -a -n 2>&1 |")) {
     # not too bad if failing on some architecture, with some luck should be alright
   } else {
@@ -334,10 +349,13 @@ sub sarun {
   $scrargs =~ s!/!\\!g if ($^O =~ /^MS(DOS|Win)/i);
   print ("\t$scrargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$scrargs > log/d.$testname/${Test::ntest} $post_redir");
+  
+  my $test_number = test_number();
+
+  untaint_system("$scrargs > log/d.$testname/$test_number $post_redir");
   $sa_exitcode = ($?>>8);
   if ($sa_exitcode != 0) { return undef; }
-  &checkfile ("d.$testname/${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/$test_number", $read_sub) if (defined $read_sub);
   1;
 }
 
@@ -368,10 +386,13 @@ sub salearnrun {
   $salearnargs =~ s!/!\\!g if ($^O =~ /^MS(DOS|Win)/i);
   print ("\t$salearnargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$salearnargs > log/d.$testname/${Test::ntest}");
+
+  my $test_number = test_number();
+
+  untaint_system("$salearnargs > log/d.$testname/$test_number");
   $salearn_exitcode = ($?>>8);
   if ($salearn_exitcode != 0) { return undef; }
-  &checkfile ("d.$testname/${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/$test_number", $read_sub) if (defined $read_sub);
   1;
 }
 
@@ -412,10 +433,13 @@ sub spamcrun {
 
   print ("\t$spamcargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
+
+  my $test_number = test_number();
+
   if ($capture_stderr) {
-    system ("$spamcargs > log/d.$testname/out.${Test::ntest} 2>&1");
+    untaint_system ("$spamcargs > log/d.$testname/out.$test_number 2>&1");
   } else {
-    system ("$spamcargs > log/d.$testname/out.${Test::ntest}");
+    untaint_system ("$spamcargs > log/d.$testname/out.$test_number");
   }
 
   $sa_exitcode = ($?>>8);
@@ -425,7 +449,7 @@ sub spamcrun {
 
   %found = ();
   %found_anti = ();
-  &checkfile ("d.$testname/out.${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/out.$test_number", $read_sub) if (defined $read_sub);
 
   if ($expect_failure) {
     ($sa_exitcode != 0);
@@ -455,7 +479,9 @@ sub spamcrun_background {
 
   print ("\t$spamcargs &\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$spamcargs > log/d.$testname/bg.${Test::ntest} &") and return 0;
+  
+  my $test_number = test_number();
+  untaint_system ("$spamcargs > log/d.$testname/bg.$test_number &") and return 0;
 
   1;
 }
@@ -527,10 +553,12 @@ sub start_spamd {
   }
 
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  my $spamd_stdout = "log/d.$testname/spamd.out.${Test::ntest}";
-     $spamd_stderr = "log/d.$testname/spamd.err.${Test::ntest}";    #  global
-  my $spamd_stdlog = "log/d.$testname/spamd.log.${Test::ntest}";
-
+  
+  my $test_number = test_number();
+  my $spamd_stdout = "log/d.$testname/spamd.out.$test_number";
+     $spamd_stderr = "log/d.$testname/spamd.err.$test_number";    #  global
+  my $spamd_stdlog = "log/d.$testname/spamd.log.$test_number";
+  my $spamd_pidfile = "log/spamd.pid";
   my $spamd_forker = $ENV{'SPAMD_FORKER'}   ?
                        $ENV{'SPAMD_FORKER'} :
                      $RUNNING_ON_WINDOWS    ?
@@ -546,36 +574,44 @@ sub start_spamd {
                        @spamd_args,
                        $spamd_extra_args,
                        qq{-s ${spamd_stderr}.timestamped},
+                       qq{-r ${spamd_pidfile}},
                        qq{&},
                     );
 
   # DEBUG instrumentation to trace spamd processes. See bug 5731 for history
   # if (-f "/home/jm/capture_spamd_straces") {
-  # $spamd_cmd = "strace -ttt -fo log/d.$testname/spamd.strace.${Test::ntest} $spamd_cmd";
+  # $spamd_cmd = "strace -ttt -fo log/d.$testname/spamd.strace.$test_number $spamd_cmd";
   # }
 
-  unlink ($spamd_stdout, $spamd_stderr, $spamd_stdlog);
+  unlink ($spamd_stdout, $spamd_stderr, $spamd_stdlog, $spamd_pidfile);
   print ("\t${spamd_cmd}\n");
   my $startat = time;
-  system ($spamd_cmd);
+  untaint_system ($spamd_cmd);
 
-  # now find the PID
   $spamd_pid = 0;
+  # Find the PID, either in the pidfile or the log... 
   # note that the wait period increases the longer it takes,
   # 20 retries works out to a total of 60 seconds
   my $retries = 30;
-  my $wait = 0;
+  my $wait = 7;
+  sleep $wait ;
   while ($spamd_pid <= 0) {
     my $spamdlog = '';
-
+    my $pidstr = untaint_cmd("cat $spamd_pidfile 2>/dev/null");
+    if ($pidstr) {
+       chomp $pidstr;
+       $spamd_pid = $pidstr;
+       dbgprint("Found PID $spamd_pid in pidfile\n");
+       last
+    }
     if (open (IN, "<${spamd_stderr}")) {
       while (<IN>) {
         # Yes, DO retry on this error. I'm getting test failures otherwise
         # /Address already in use/ and $retries = 0;
-	/server pid: (\d+)/ and $spamd_pid = $1;
+	/server pid: (\d+)/ and $spamd_pid = "$1" and dbgprint("Found PID $spamd_pid in stderr logfile\n");
 
-        if (/ERROR/) {
-          warn "spamd error! $_";
+        if ( !(/dbg: config: .*rulename/) && (/\bERROR/) ){
+          warn "spamd start failed - spamd error! $_\nExiting test with debug output";
           $retries = 0; last;
         }
 
@@ -585,12 +621,15 @@ sub start_spamd {
       last if ($spamd_pid);
     }
 
-    sleep (int($wait++ / 4) + 1) if $retries > 0;
+    my $sleep = (int($wait++ / 4) + 1);
+    warn "spam_pid not found: Sleeping $sleep - Retry # $retries\n";
+
+    sleep $sleep if $retries > 0;
 
     if ($retries-- <= 0) {
-      warn "spamd start failed: log: $spamdlog";
+      warn "spamd start failed - Could not find a valid PID.\nEnd Debug log -------------------\n$spamdlog\nEnd Debug log -------------------";
       warn "\n\nMaybe you need to kill a running spamd process?\n";
-      warn "started at $startat, gave up at ".time."\n\n";
+      warn "Or the start took too long. Started at $startat, gave up at ".time."\n\n";
       return 0;
     }
   }
@@ -603,6 +642,7 @@ sub stop_spamd {
   die "NO_SPAMD_REQUIRED in stop_spamd! oops" if $NO_SPAMD_REQUIRED;
 
   $spamd_pid ||= 0;
+  $spamd_pid = untaint_var($spamd_pid);
   if ( $spamd_pid <= 1) {
     print ("Invalid spamd pid: $spamd_pid. Spamd not started/crashed?\n");
     return 0;
@@ -844,7 +884,7 @@ sub read_config {
     my $v = shift @ARGV;
 
     # Override only allows setting one variable.  Some xt tests need to set more
-    # config variables.  Adding : as a delimeter for config variable and value 
+    # config variables.  Adding : as a delimiter for config variable and value 
     # parameters
 
     @k = split (/:/,$k);
@@ -964,6 +1004,7 @@ sub read_from_pidfile {
     $npid = <PID>;
     if (defined $npid) { chomp $npid; }
     close(PID);
+    $npid = untaint_var($npid);
 
     if (!$npid || $npid < 1) {
       warn "failed to read anything sensible from $f, retrying read";
@@ -983,13 +1024,13 @@ sub read_from_pidfile {
 sub system_or_die {
   my $cmd = $_[0];
   print ("\t$cmd\n");
-  system($cmd);
+  untaint_system($cmd);
   $? == 0  or die "'$cmd' failed: ".exit_status_str($?,0);
 }
 
 # (sub exit_status_str copied from Util.pm)
 # map process termination status number to an informative string, and
-# append optional mesage (dual-valued errno or a string or a number),
+# append optional message (dual-valued errno or a string or a number),
 # returning the resulting string
 #
 sub exit_status_str($;$) {
@@ -1023,7 +1064,7 @@ sub can_use_net_dns_safely {
   # (which is used by Net::DNS)
 
   return 1 if ($< != 0);
-  return 1 if ($^O =~ /^(linux|mswin|dos|os2)/oi);
+  return 1 if ($^O =~ /^(linux|mswin|dos|os2|openbsd)/oi);
 
   my $has_unsafe_hostname =
     eval { require Sys::Hostname::Long && Sys::Hostname::Long->VERSION < 1.4 };
@@ -1068,4 +1109,33 @@ sub debug_array {
   return $string;
 }
 
+sub test_number {
+  return Test::More->builder->current_test;
+}
+
+# Simple version of untaint_var for internal use
+sub untaint_var {
+    local($1);
+    $_[0] =~ /^(.*)\z/s;
+    return $1;
+}
+
+# untainted system()
+sub untaint_system {
+    my @args;
+    push @args, untaint_var($_) foreach (@_);
+    return system(@args);
+}
+
+# untainted version of `shell command`
+sub untaint_cmd {
+    if (open(CMD, untaint_var($_[0])."|")) {
+      my $stdout = do { local($/); <CMD> };
+      close CMD;
+      return $stdout;
+    } else {
+      return "";
+    }
+}
+
 1;
